searchData={"items":[{"type":"module","title":"MontgomeryReduction","doc":"A library for efficient modular arithmetic using Montgomery reduction.\n\nMontgomery reduction is a technique for efficiently computing modular arithmetic\noperations, particularly useful in cryptographic applications where large modular\nmultiplications are performed frequently.","ref":"MontgomeryReduction.html"},{"type":"module","title":"Overview - MontgomeryReduction","doc":"Montgomery reduction allows for efficient computation of modular arithmetic by\ntransforming the problem into a different domain where modular reduction is\nperformed using simple bit shifts and additions instead of expensive division\noperations.","ref":"MontgomeryReduction.html#module-overview"},{"type":"module","title":"Key Features - MontgomeryReduction","doc":"* **Efficient modular arithmetic**: Perform modular operations without expensive division\n* **Cryptographic applications**: Optimized for large number operations common in cryptography\n* **Configurable radix**: Support for different Montgomery radix sizes\n* **Error handling**: Comprehensive validation and error reporting","ref":"MontgomeryReduction.html#module-key-features"},{"type":"module","title":"Mathematical Background - MontgomeryReduction","doc":"Montgomery reduction computes $tR^{-1} \\mod n$ where:\n- $R = 2^{r\\_bits}$ is the Montgomery radix (typically a power of 2)\n- $R^{-1}$ is the modular multiplicative inverse of R modulo n\n- $n$ is the modulus (must be odd)\n\nThis transformation allows modular multiplication to be performed using only\nadditions, multiplications, and bit shifts, making it much more efficient\nthan traditional modular arithmetic for large numbers.","ref":"MontgomeryReduction.html#module-mathematical-background"},{"type":"module","title":"Usage - MontgomeryReduction","doc":"iex> reducer = MontgomeryReduction.of(17)\n    iex> reducer.(100)\n    15\n\n    iex> reducer = MontgomeryReduction.of(23, 8)\n    iex> reducer.(50)\n    9","ref":"MontgomeryReduction.html#module-usage"},{"type":"module","title":"Performance - MontgomeryReduction","doc":"Montgomery reduction is particularly beneficial when:\n- Working with large numbers (typically 256+ bits)\n- Performing many modular operations\n- Implementing cryptographic algorithms like RSA or elliptic curve cryptography","ref":"MontgomeryReduction.html#module-performance"},{"type":"module","title":"Dependencies - MontgomeryReduction","doc":"This library depends on:\n- `binary_gcd` for efficient GCD calculations\n- `nstandard` for development tooling","ref":"MontgomeryReduction.html#module-dependencies"},{"type":"module","title":"References - MontgomeryReduction","doc":"* [Montgomery, P.L. (1985). Modular multiplication without trial division](https://doi.org/10.1090/S0025-5718-1985-0777282-X)\n* [Handbook of Applied Cryptography - Chapter 14](https://cacr.uwaterloo.ca/hac/)","ref":"MontgomeryReduction.html#module-references"},{"type":"function","title":"MontgomeryReduction.of/2","doc":"Creates a Montgomery reduction function for the given modulus `n`.\n\nMontgomery reduction is a technique for efficiently computing modular arithmetic\noperations. This function returns a closure that can perform Montgomery reduction\non values.","ref":"MontgomeryReduction.html#of/2"},{"type":"function","title":"Parameters - MontgomeryReduction.of/2","doc":"* `n` - The modulus (must be odd)\n  * `r_bits` - The number of bits for the Montgomery radix (default: 256)","ref":"MontgomeryReduction.html#of/2-parameters"},{"type":"function","title":"Returns - MontgomeryReduction.of/2","doc":"A function that takes a value `t` and returns the Montgomery reduction of `t` modulo `n`.","ref":"MontgomeryReduction.html#of/2-returns"},{"type":"function","title":"Examples - MontgomeryReduction.of/2","doc":"iex> reducer = MontgomeryReduction.of(17)\n    iex> reducer.(100)\n    15\n\n    iex> reducer = MontgomeryReduction.of(23, 8)\n    iex> reducer.(50)\n    9","ref":"MontgomeryReduction.html#of/2-examples"},{"type":"function","title":"Error Cases - MontgomeryReduction.of/2","doc":"The function will raise a `ArithmeticError` if:\n\n  * The modulus `n` is even (Montgomery reduction requires odd moduli)\n  * The input value `t` is greater than or equal to $n 2^{r_{bits}}$","ref":"MontgomeryReduction.html#of/2-error-cases"},{"type":"function","title":"Mathematical Background - MontgomeryReduction.of/2","doc":"Montgomery reduction computes $tR^{-1} \\mod n$ where:\n- $R = 2^{r\\_bits}$ is the Montgomery radix\n- $R^{-1}$ is the modular multiplicative inverse of R modulo n\n\nThis is useful for efficient modular arithmetic, especially in cryptographic applications.","ref":"MontgomeryReduction.html#of/2-mathematical-background"},{"type":"extras","title":"Montgomery Reduction","doc":"# Montgomery Reduction\n\nA high-performance Elixir library for efficient modular arithmetic using Montgomery reduction.\n\n[![Hex.pm](https://img.shields.io/hexpm/v/montgomery_reduction.svg)](https://hex.pm/packages/montgomery_reduction)\n[![Hex.pm](https://img.shields.io/hexpm/dt/montgomery_reduction.svg)](https://hex.pm/packages/montgomery_reduction)\n[![Documentation](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/montgomery_reduction)","ref":"readme.html"},{"type":"extras","title":"Overview - Montgomery Reduction","doc":"Montgomery reduction is a technique for efficiently computing modular arithmetic operations, particularly useful in cryptographic applications where large modular multiplications are performed frequently. This library provides a clean, efficient implementation of Montgomery reduction in Elixir.","ref":"readme.html#overview"},{"type":"extras","title":"Features - Montgomery Reduction","doc":"- **Efficient modular arithmetic**: Perform modular operations without expensive division\n- **Cryptographic applications**: Optimized for large number operations common in cryptography\n- **Configurable radix**: Support for different Montgomery radix sizes\n- **Error handling**: Comprehensive validation and error reporting\n- **Type specifications**: Full type specs for better development experience","ref":"readme.html#features"},{"type":"extras","title":"Installation - Montgomery Reduction","doc":"Add `montgomery_reduction` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:montgomery_reduction, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Montgomery Reduction","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - Montgomery Reduction","doc":"```elixir\n# Create a Montgomery reducer for modulus 17\nreducer = MontgomeryReduction.of(17)\n\n# Perform Montgomery reduction\nresult = reducer.(100)\n# => 15\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Custom Radix - Montgomery Reduction","doc":"```elixir\n# Create a reducer with 8-bit radix\nreducer = MontgomeryReduction.of(23, 8)\nresult = reducer.(50)\n# => 9\n```","ref":"readme.html#custom-radix"},{"type":"extras","title":"Error Handling - Montgomery Reduction","doc":"The library provides comprehensive error handling:\n\n```elixir\n# This will raise an error - modulus must be odd\nMontgomeryReduction.of(16)\n\n# This will raise an error - input too large\nreducer = MontgomeryReduction.of(17)\nreducer.(1000000)  # Input exceeds n * 2^r_bits\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Mathematical Background - Montgomery Reduction","doc":"Montgomery reduction computes $tR^{-1} \\mod n$ where:\n- $R = 2^{r\\_bits}$ is the Montgomery radix (typically a power of 2)\n- $R^{-1}$ is the modular multiplicative inverse of R modulo n\n- $n$ is the modulus (must be odd)\n\nThis transformation allows modular multiplication to be performed using only additions, multiplications, and bit shifts, making it much more efficient than traditional modular arithmetic for large numbers.","ref":"readme.html#mathematical-background"},{"type":"extras","title":"Performance - Montgomery Reduction","doc":"Montgomery reduction is particularly beneficial when:\n- Working with large numbers (typically 256+ bits)\n- Performing many modular operations\n- Implementing cryptographic algorithms like RSA or elliptic curve cryptography","ref":"readme.html#performance"},{"type":"extras","title":"Dependencies - Montgomery Reduction","doc":"This library depends on:\n- `binary_gcd` for efficient GCD calculations\n- `nstandard` for development tooling","ref":"readme.html#dependencies"},{"type":"extras","title":"License - Montgomery Reduction","doc":"Copyright (c) 2025 University of Kitakyushu\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"References - Montgomery Reduction","doc":"- [Montgomery, P.L. (1985). Modular multiplication without trial division](https://doi.org/10.1090/S0025-5718-1985-0777282-X)","ref":"readme.html#references"},{"type":"extras","title":"Documentation - Montgomery Reduction","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc) and published on [HexDocs](https://hexdocs.pm). Once published, the docs can be found at  .","ref":"readme.html#documentation"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.0.0] - 2025-07-19","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- **Initial Release**: Complete Montgomery reduction implementation\n- **Core Functionality**: `MontgomeryReduction.of/2` function with configurable radix\n- **Input Validation**: Comprehensive error handling for invalid moduli and inputs\n- **Bitwise Optimizations**: Efficient implementation using bitwise operations\n- **Pattern Matching**: Proper function clauses with guard conditions\n- **Documentation**: Complete module and function documentation with examples\n- **Type Specifications**: Full type specs for better development experience\n- **Project Setup**: Initial project setup with comprehensive development tooling","ref":"changelog.html#added"},{"type":"extras","title":"Features - Changelog","doc":"- Support for custom Montgomery radix sizes (default: 256 bits)\n- Efficient modular arithmetic without expensive division operations\n- Optimized for cryptographic applications with large numbers\n- Comprehensive error messages for debugging\n- Mathematical background and performance considerations in documentation","ref":"changelog.html#features"},{"type":"extras","title":"Technical Details - Changelog","doc":"- Uses `ModularInverse` for efficient n_prime calculation\n- Implements proper Montgomery reduction algorithm with bitwise operations\n- Supports odd modulus requirement with validation\n- Handles overflow conditions with appropriate error messages\n- Returns function closure for repeated reduction operations","ref":"changelog.html#technical-details"},{"type":"extras","title":"Documentation - Changelog","doc":"- Comprehensive README with usage examples and mathematical background\n- Complete API documentation with parameter descriptions\n- Performance considerations for cryptographic applications\n- Academic references and citations\n- Contributing guidelines and license information","ref":"changelog.html#documentation"},{"type":"extras","title":"Development - Changelog","doc":"- Full development tooling setup with code quality checks\n- Comprehensive test suite\n- Documentation generation with ExDoc\n- Static analysis with Dialyzer\n- Spell checking and code formatting","ref":"changelog.html#development"},{"type":"extras","title":"Dependencies - Changelog","doc":"- Added `binary_gcd` for efficient GCD calculations\n- Added `modular_inverse` for modular multiplicative inverse operations\n- Added `nstandard` for development tooling\n- Added `ex_doc` for documentation generation\n- Added `dialyxir` for static analysis\n- Added `credo` for code quality checks\n- Added `spellweaver` for spell checking\n\n---","ref":"changelog.html#dependencies"},{"type":"extras","title":"Version History - Changelog","doc":"","ref":"changelog.html#version-history"},{"type":"extras","title":"1.0.0 (Initial Release) - Changelog","doc":"- Complete Montgomery reduction implementation\n- Comprehensive documentation and examples\n- Full development tooling setup\n- Ready for production use in cryptographic applications\n\n---","ref":"changelog.html#1-0-0-initial-release"},{"type":"extras","title":"Contributing - Changelog","doc":"When contributing to this project, please update this changelog to reflect your changes. Follow the format established above and include:\n\n- **Added**: for new features\n- **Changed**: for changes in existing functionality\n- **Deprecated**: for soon-to-be removed features\n- **Removed**: for now removed features\n- **Fixed**: for any bug fixes\n- **Security**: in case of vulnerabilities","ref":"changelog.html#contributing"},{"type":"extras","title":"Links - Changelog","doc":"- [GitHub Repository](https://github.com/your-username/montgomery_reduction)\n- [Hex Package](https://hex.pm/packages/montgomery_reduction)\n- [Documentation](https://hexdocs.pm/montgomery_reduction)","ref":"changelog.html#links"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}